##----------------------------------------------------------------------------##
##=Purpose: Exercise 4.1 of Beyond Classic Designs & Analysis for Health      ##
##          Data in Singapore (27 June to 8 July 2022)                        ##
##
##=Data required: 
##  - Framingham_HDL.dta, 
##  - Hpylori_extract.dta
##
##=R packages/libraries required:
##  - dplyr: 
##    * for processing data
##  - haven:
##    * read_dta: To read in recent versions of STATA dta files
##  - knitr:
##    * kable: Generate nice tables
##  - sandwich:
##    * vcovHC: For robust variance 
##  - survey:
##    * twophase: Specify the two stage design information
##    * svyglm: Perform glm analysis with sampling weights
##
##=Created by:
##  - Francesca Graziano 
##  - Zilu Feng
##  - Zihan 
##  - Chuen Seng Tan
##  - Marie Reilly
##
##=Changes (if any):
##  -DDMMYYYY: Describe
##
##----------------------------------------------------------------------------##

##-Set the working directory to the data folder where the datasets are saved:
setwd("D:/nbox/Education/Marie_short_course/Singapore course 2022/20220602/exercises/data")

##-R libraries to install:
#install.packages(c("dplyr","haven","knitr","sandwich","survey"))
##-Load the R libraries
library(dplyr)
library(haven)
library(knitr)
library(sandwich)
library(survey)


#Useful self-written function to generate results from logistic regression:
#' Fit a logistic regression (unweighted) and output a table of the estimated
#' OR, the 95% CI and the p-value
#' @param ... Input to the function `glm`, i.e., specify input to this function
#'   in the same way as to function `glm`.
#' @return Returns the fitted model and prints the table of estimated OR, 95% CI
#'   and p-value. The coefficient matrix with OR is saved as an element of the 
#'   fitted model with the name `coef_with_or`. 
fit_logistic <- function(...) {
  model <- glm(...)
  tmp <- as.data.frame(summary(model)$coef)
  tmp$OR <- exp(tmp[,1])
  tmp$lower.CI <- exp(tmp[,1] - 1.96 * tmp[,2])
  tmp$upper.CI <- exp(tmp[,1] + 1.96 * tmp[,2])
  names(tmp)[c(2, 4)] <- c('SE(log.OR)', 'p-value')
  model$coef_with_or <- tmp
  print(knitr::kable(tmp[,c('OR','SE(log.OR)','lower.CI','upper.CI','p-value')],
                     digits = 3, format = "pandoc", 
                     col.names = c("OR", "SE(log.OR)", "Lower 95% CI",
                                   "Upper 95% CI", "p-value"), 
                     caption = "Logistic regression using glm"))
  model
}


################################################################################
##                               Question 1      
##
##-1. In this exercise, you will sample a two-stage study from a cross-sectional 
##    of individuals from the Framingham cohort study, whose data are in the 
##    dataset Framingham_HDL.dta provided.  This data consists of 3,026 cohort 
##    members who had HDL and LDL measurements taken at their third visit. The 
##    outcome of interest is any CHD, with 777 cases and 2249 non-cases. Our 
##    objective is to estimate the effect of HDL and LDL on any CHD, adjusted 
##    for sex and hypertension. The variables are:
##
##    * randid (individual identification number)
##    * anychd (case indicator: 0=no CHD during follow-up, 1=CHD during follow-up)
##    * sex (1=male, 2=female)
##    * cursmoke (0=not current smoker, 1=current smoker)
##    * prevhyp (0=no prevalent hypertension, 1=prevalent hypertension)
##    * ldlc (low-density lipoprotein cholesterol, mg/dL) 
##    * hdlc (high-density lipoprotein cholesterol, mg/dL) 
##    * ldlc_cat (categorical LDL: desirable = <=100mg/dL, 
##                moderate = 101-159mg/dL, high = 160-189mg/dL, 
##                very high = >=190mg/dL)
##    * hdlc_cat (categorical HDL: low = <=40mg/dL, borderline = 41-59mg/dL, 
##                desirable = >=60mg/dL)
##    * hdlc_cat2 (hdlc_cat, releveled so 'desirable' is the reference) 
################################################################################

################################################################################
##                               Question 1 (i)     
##
##-(i) Run a logistic regression on the full cross-sectional sample with anychd 
##     as outcome and covariates ldlc_cat, hdlc_cat2, sex, and prevhyp, to get 
##     adjusted ORs for HDL and LDL.
################################################################################

#1- read in the data 
full_hdl <- haven::read_dta("Framingham_HDL.dta")

#2- declare ldlc_cat, hdlc_cat2 and sex as factors:
full_hdl$ldlc_cat <- factor(full_hdl$ldlc_cat,
                            levels = c(1,2,3,4),
                            labels = c("Desirable", "Moderate", "High", "Very High"))
full_hdl$hdlc_cat2 <- factor(full_hdl$hdlc_cat2,
                             levels = c(1,2,3),
                             labels = c("Desirable", "Borderline", "Low"))
full_hdl$sex <- factor(full_hdl$sex,
                       levels = c(1,2),
                       labels = c("Male", "Female"))


#3- fit logistic regression model to full dataset
logit_whole <- fit_logistic(anychd ~ ldlc_cat + hdlc_cat2 + sex + prevhyp, 
                            family = binomial(link = "logit"), 
                            data = full_hdl)

################################################################################
##                               Question 1 (ii)     
##
##-(ii) Now we will mimic a two-stage study where HDL and LDL were considered 
##      as 'expensive' covariates and were only measured for a second-stage 
##      subsample and were missing for the other individuals. To mimic a 
##      situation where the second-stage sample consists of all 777 cases and 
##      an equal number of controls, select/identify a subsample of 777 
##      controls (using a random number generator) and save a dataset 
##      containing all the cases and these controls (Note. You should set a 
##      "seed" so that you can reproduce the exact same data if needed).
################################################################################

#1 - extract all cases
all.cases <- full_hdl %>%
  dplyr::filter(anychd == 1) 
no.controls<-nrow(all.cases)

#2- sample the controls
set.seed(2307)
sampled.controls <- full_hdl %>%
  dplyr::filter(anychd == 0)  %>% 
  dplyr::slice_sample(n=no.controls, replace = FALSE)

#3 -combine the sampled controls and all cases
dat.2s <- rbind(sampled.controls,
                all.cases)


################################################################################
##                               Question 1 (iii)     
##
##-(iii) For the dataset selected in (ii), create a weight variable that is 1 
##       for all cases and 2249/777 for each of the controls. Verify that 
##       running a weighted logistic regression (with robust SE) of just these 
##       (second-stage) data gives similar estimates to the full cohort results 
##       in (i).
################################################################################

#1- assign weights
sum(full_hdl$anychd)
dat.2s <- dat.2s %>%
  dplyr::mutate(weights=ifelse(anychd==1,1, 2249/777))

#2- performed weighted logistic regression using only the second-stage sample 
weighted_model <- glm(anychd ~ ldlc_cat + hdlc_cat2 + sex + prevhyp, 
                      family = binomial("logit"), 
                      data = dat.2s, 
                      weights = weights)

cov.m1 <- vcovHC(weighted_model, type = "HC0")
std.err <- sqrt(diag(cov.m1))
q.val <- qnorm(0.975)
r.est <- cbind(
  OR = round(exp(coef(weighted_model)),3)
  , "Robust SE(log.OR)" = round(std.err,3)
  , "Lower 95% CI" = round(exp(coef(weighted_model) - q.val  * std.err),3)
  , "Upper 95% CI" = round(exp(coef(weighted_model) + q.val  * std.err),3)
  , "Pr(>|z|) "= round(2 * pnorm(abs(coef(weighted_model)/std.err), lower.tail = FALSE),3))
knitr::kable(r.est, format = "pandoc",
             col.names = c("OR", "Robust SE(log.OR)", "Lower 95% CI", "Upper 95% CI", "p-value"))

#this gives output which has similar estimates in the full cohort.


################################################################################
##                               Question 1 (iv)     
##
##-(iv) Return to the full data set, and this time when you select the 777 
##      controls, do not delete the others from the data, but set their HDL and 
##      LDL variables to missing, and add a second-stage indicator variable to 
##      the data which is 0 for these missing observations and 1 for the cases 
##      and 777 controls. Repeat the logistic regression analysis this data set 
##      using the meanscor command in Stata or the survey package in R and 
##      compare your results to what you obtained with weighted regression in (iii).
################################################################################

#1- generate data with stage2 variable to indicate selection into the 2-stage
#   with randid variable only
dat.2s.select <- dat.2s %>%
  dplyr::mutate(stage2=1) %>%
  dplyr::select(randid,stage2)

#2- merge the previous data with the full data
dat.2s.full <- merge(x = full_hdl, y = dat.2s.select, 
                    by = "randid", all.x = TRUE)

#3- replace NA entries of stage2 variable as 0 and their correspodning HDL and 
#   LDL entries as missing
dat.2s.full <- dat.2s.full %>%
  dplyr::mutate(stage2=replace(stage2,is.na(stage2),0)) %>% 
  dplyr::mutate(ldlc_cat=replace(ldlc_cat,stage2==0,NA)) %>% 
  dplyr::mutate(hdlc_cat2=replace(hdlc_cat2,stage2==0,NA)) 

#4- specify study design as required by "survey" library
dat.2s.full.design <- survey::twophase(id = list(~randid, ~randid), 
                                       subset = ~stage2, 
                                       strata = list(NULL, ~anychd), 
                                       data = dat.2s.full)

#5- run model
dat.2s.full.svy <- survey::svyglm(anychd ~ ldlc_cat + hdlc_cat2 + sex + prevhyp, 
                                  family = binomial("logit"), 
                                  design = dat.2s.full.design)

tmp1 <- data.frame(summary(dat.2s.full.svy)$coef)
tmp1$OR <- round(exp(tmp1[,1]),3)
tmp1$SE.log.OR <- round(tmp1[,2],3)
tmp1$lower.CI <- round(exp(tmp1[,1] - 1.96* tmp1[,2]),3)
tmp1$upper.CI <- round(exp(tmp1[,1] + 1.96* tmp1[,2]),3)
names(tmp1)[4] <- 'p-value'
knitr::kable(tmp1[,c('OR','SE.log.OR', 'lower.CI','upper.CI','p-value')])  

#this gives output that is similar as the weighted logistic regression in (iii).


################################################################################
##                               Question 2    
##
##-2. This exercise uses some of the data collected in a cross-sectional survey 
##    of schoolchildren in Stockholm schools to determine the risk of H. Pylori 
##    infection in children from infected family members. For all study 
##    children, information was gathered on household socioeconomic status (SES), 
##    and for immigrant children, whether the family's country of origin had a high 
##    or low H. pylori prevalence. The information on H. Pylori infection in 
##    family members was only collected for a sub-sample of children. The 
##    family members of all infected children were tested, but in an effort to 
##    target the control families considered most informative, the intensity of 
##    sampling was different in the strata defined by SES (high vs. Low) and 
##    the H. Pylori prevalence (high vs. Low) in the child's 
##    country of origin. Thus the second-stage sampling is stratified, with 
##    6 strata defined by case/control status, SES (high/low) and 
##    prevalence (high/low).
##
##    The data set provided, Hpylori_extract.dta, is an extract from the full 
##    data, containing only children from low prevalence countries, with the 
##    following information: H. Pylori (HP) status of the child (hpproband), 
##    HP status of the mother (hpmother), SES category (sescat) and an indicator 
##    variable (stage2) for whether the mother's HP infection status is known.
##
##    Use a two-stage analysis to estimate the effect of a HP-positive mother 
##    on the child's risk, adjusted for SES. (Note that since we are 
##    investigating only children from low-prevalence countries, the 
##    second-stage data is in 4 strata defined by case/control and high/low SES). 
##    Compare your results to those from the more detailed model in the Table 
##    below, where the full data set was analysed using a weighted regression 
##    (the coloumn "complete" refers to a simple logistic regression of the 
##    complete (i.e. stage 2) data, i.e. only children whose family members 
##    were tested.
##  
##    [see exercise for the table]
################################################################################

#1- read in data
hpy <- haven::read_dta("Hpylori_extract.dta")
hpy$id <- 1:nrow(hpy)

#2- create 4 strata, defined by case/control status, SES level
hpy$strata_cat <- interaction(hpy$hpproband, hpy$sescat)

#3- run weighted logistic regression using survey library by specifying the design
hpy.design <- survey::twophase(id = list(~id, ~id), 
                               subset = ~stage2, 
                               strata = list(NULL, ~strata_cat), 
                               data = hpy)

#4- run weighted logistic regression using survey 
hpy.svy <- survey::svyglm(hpproband ~ sescat + hpmother, 
                          family = binomial("logit"), 
                          design = hpy.design)

tmp2 <- data.frame(summary(hpy.svy)$coef)
tmp2$OR <- round(exp(tmp2[,1]),3)
tmp2$SE.log.OR <- round(tmp2[,2],3)
tmp2$lower.CI <- round(exp(tmp2[,1] - 1.96* tmp2[,2]),3)
tmp2$upper.CI <- round(exp(tmp2[,1] + 1.96* tmp2[,2]),3)
names(tmp2)[4] <- 'p-value'
knitr::kable(tmp2[,c('OR','SE.log.OR', 'lower.CI','upper.CI','p-value')])

#5- run weighted logistic regression by using 2-stage data only with weights 
#   without using survey library by generating the data:
table(hpy$strata_cat,hpy$stage2)
hpy.2s <- hpy %>%
  dplyr::filter(stage2==1) %>% 
  dplyr::mutate(weights=(240+36)/36) %>% 
  dplyr::mutate(weights=replace(weights,strata_cat=="1.0",1)) %>% 
  dplyr::mutate(weights=replace(weights,strata_cat=="0.1",(177+55)/55)) %>% 
  dplyr::mutate(weights=replace(weights,strata_cat=="1.1",(3+34)/34))

#6- performed weighted logistic regression without using the survey library 
hpy.2s.wt <- glm(hpproband ~ sescat + hpmother, 
                 family = binomial("logit"), 
                 data = hpy.2s, 
                 weights = weights)

cov.m1 <- vcovHC(hpy.2s.wt, type = "HC0")
std.err <- sqrt(diag(cov.m1))
q.val <- qnorm(0.975)
r.est <- cbind(
  OR = round(exp(coef(hpy.2s.wt)),3)
  , "Robust SE(log.OR)" = round(std.err,3)
  , "Lower 95% CI" = round(exp(coef(hpy.2s.wt) - q.val  * std.err),3)
  , "Upper 95% CI" = round(exp(coef(hpy.2s.wt) + q.val  * std.err),3)
  , "Pr(>|z|) "= round(2 * pnorm(abs(coef(hpy.2s.wt)/std.err), lower.tail = FALSE),3))
knitr::kable(r.est, format = "pandoc",
             col.names = c("OR", "Robust SE(log.OR)", "Lower 95% CI", "Upper 95% CI", "p-value"))

#The estimates are on the similar direction with comparable findings but SES is significant in the exericse not in the table.

################################################################################
##                               Question 3(i)    
##
##-(i) Repeat the analysis in Question 1 using a two-stage case-control study 
##     with 777 randomly sampled men and 777 randomly sampled women selected at 
##     the second stage. 
################################################################################

#1 - sample the 777 males and females using data from question 1
set.seed(2307)
dat.2s.sex <- full_hdl %>%
  dplyr::group_by(sex) %>% 
  dplyr::slice_sample(n=777, replace = FALSE)

#2- assign weights for analysis not using survey library
table(full_hdl$sex)
dat.2s.sex <- dat.2s.sex %>%
  dplyr::mutate(weights=ifelse(sex=="Male",1304/777,1722/777))


#3- performed weighted logistic regression not using survey library 
dat.2s.sex.wt <- glm(anychd ~ ldlc_cat + hdlc_cat2 + sex + prevhyp, 
                      family = binomial("logit"), 
                      data = dat.2s.sex, 
                      weights = weights)

cov.m1 <- vcovHC(dat.2s.sex.wt, type = "HC0")
std.err <- sqrt(diag(cov.m1))
q.val <- qnorm(0.975)
r.est <- cbind(
  OR = round(exp(coef(dat.2s.sex.wt)),3)
  , "Robust SE(log.OR)" = round(std.err,3)
  , "Lower 95% CI" = round(exp(coef(dat.2s.sex.wt) - q.val  * std.err),3)
  , "Upper 95% CI" = round(exp(coef(dat.2s.sex.wt) + q.val  * std.err),3)
  , "Pr(>|z|) "= round(2 * pnorm(abs(coef(dat.2s.sex.wt)/std.err), lower.tail = FALSE),3))
knitr::kable(r.est, format = "pandoc",
             col.names = c("OR", "Robust SE(log.OR)", "Lower 95% CI", "Upper 95% CI", "p-value"))

#4- generate data with stage2 variable to indicate selection into the 2-stage
#   with randid variable only
dat.2s.sex.select <- dat.2s.sex %>%
  dplyr::mutate(stage2=1) %>%
  dplyr::select(randid,stage2,sex) 

#5- merge the previous data with the full data
dat.2s.sex.full <- merge(x = full_hdl, y = dat.2s.sex.select[,-3], 
                     by = "randid", all.x = TRUE)

#6- replace NA entries of stage2 variable as 0 and their correspodning HDL and 
#   LDL entries as missing
dat.2s.sex.full <- dat.2s.sex.full %>%
  dplyr::mutate(stage2=replace(stage2,is.na(stage2),0)) %>% 
  dplyr::mutate(ldlc_cat=replace(ldlc_cat,stage2==0,NA)) %>% 
  dplyr::mutate(hdlc_cat2=replace(hdlc_cat2,stage2==0,NA)) 

#7- specify study design as required by "survey" library
dat.2s.sex.full.design <- survey::twophase(id = list(~randid, ~randid), 
                                       subset = ~stage2, 
                                       strata = list(NULL, ~sex), 
                                       data = dat.2s.sex.full)

#8- run model
dat.2s.sex.full.svy <- survey::svyglm(anychd ~ ldlc_cat + hdlc_cat2 + sex + prevhyp, 
                                  family = binomial("logit"), 
                                  design = dat.2s.sex.full.design)

tmp1 <- data.frame(summary(dat.2s.sex.full.svy)$coef)
tmp1$OR <- round(exp(tmp1[,1]),3)
tmp1$SE.log.OR <- round(tmp1[,2],3)
tmp1$lower.CI <- round(exp(tmp1[,1] - 1.96* tmp1[,2]),3)
tmp1$upper.CI <- round(exp(tmp1[,1] + 1.96* tmp1[,2]),3)
names(tmp1)[4] <- 'p-value'
knitr::kable(tmp1[,c('OR','SE.log.OR', 'lower.CI','upper.CI','p-value')])  


################################################################################
##                               Question 3(ii)    
##
##-(ii) Repeat the two-stage sampling experiment again but this time taking a 
##      second-stage sample of 1554 (i.e. 2 X 777) that is "balanced" across 
##      the eight strata defined by the anychd, sex and prevhyp (i.e. equal 
##      numbers in each stratum) using the following two approaches:
##      a. Select one-eight of the 1554 (195) from each stratum (or all, if a 
##         stratum has fewer)
##      b. Strategy a. will result in fewer than 1554. Select the "surplus" from 
##         the strata larger than 195 to yield a sample of size 1554 that is as 
##         "balanced" as possible
##
##      Re-run the weighted logistic regression for second-stage data for these 
##      two designs, using the appropriate weights. Compare your estimates to 
##      those in Question 1. 
##
##      Sampling strategy b. is just one way of allocating "surplus" records to 
##      achieve a given total sample size. However, these could have been chosen 
##      from any stratum/strata, provided they are chosen randomly: explain how 
##      this would impact on the analysis and suggest why an investigator might 
##      wish to take an "unbalanced" sample.
################################################################################

#0 - generate strata_cat indicating the 8 strata
dat.full_hdl <- full_hdl %>%
  dplyr::mutate(strata_cat=interaction(anychd,sex,prevhyp))

##############---------Strategy a---------##############
#1 - sample 195 in each strata or lesser if the stratum has <195 observations 
set.seed(2307)
dat.2s.a <- dat.full_hdl %>%
  dplyr::group_by(strata_cat) %>% 
  dplyr::slice_sample(n=195, replace = FALSE)


#2- assign weights for analysis not using survey library
#2a- generate the aggregated data for each stratum from full and sampled data
cbind("full"=table(dat.full_hdl$strata_cat),
      "2stage"=table(dat.2s.a$strata_cat))

dat.2s.a <- dat.2s.a %>%
  dplyr::mutate(weights=390/195) %>%
  dplyr::mutate(weights=replace(weights,strata_cat=="1.Male.0",120/120)) %>%
  dplyr::mutate(weights=replace(weights,strata_cat=="0.Female.0",631/195)) %>%
  dplyr::mutate(weights=replace(weights,strata_cat=="1.Female.0",81/81)) %>%
  dplyr::mutate(weights=replace(weights,strata_cat=="0.Male.1",479/195)) %>%
  dplyr::mutate(weights=replace(weights,strata_cat=="1.Male.1",315/195)) %>%
  dplyr::mutate(weights=replace(weights,strata_cat=="0.Female.1",749/195)) %>%
  dplyr::mutate(weights=replace(weights,strata_cat=="1.Female.1",261/195))


#3- performed weighted logistic regression not using survey library 
dat.2s.a.wt <- glm(anychd ~ ldlc_cat + hdlc_cat2 + sex + prevhyp, 
                   family = binomial("logit"), 
                   data = dat.2s.a, 
                   weights = weights)

cov.m1 <- vcovHC(dat.2s.a.wt, type = "HC0")
std.err <- sqrt(diag(cov.m1))
q.val <- qnorm(0.975)
r.est <- cbind(
  OR = round(exp(coef(dat.2s.a.wt)),3)
  , "Robust SE(log.OR)" = round(std.err,3)
  , "Lower 95% CI" = round(exp(coef(dat.2s.a.wt) - q.val  * std.err),3)
  , "Upper 95% CI" = round(exp(coef(dat.2s.a.wt) + q.val  * std.err),3)
  , "Pr(>|z|) "= round(2 * pnorm(abs(coef(dat.2s.a.wt)/std.err), lower.tail = FALSE),3))
knitr::kable(r.est, format = "pandoc",
             col.names = c("OR", "Robust SE(log.OR)", "Lower 95% CI", "Upper 95% CI", "p-value"))

#4- generate data with stage2 variable to indicate selection into the 2-stage
#   with randid variable only
dat.2s.a.select <- dat.2s.a %>%
  dplyr::mutate(stage2=1) %>%
  dplyr::select(randid,stage2,strata_cat) 

#5- merge the previous data with the full data
dat.2s.a.full <- merge(x = dat.full_hdl, y = dat.2s.a.select[,-3], 
                         by = "randid", all.x = TRUE)

#6- replace NA entries of stage2 variable as 0 and their corresponding HDL and 
#   LDL entries as missing
dat.2s.a.full <- dat.2s.a.full %>%
  dplyr::mutate(stage2=replace(stage2,is.na(stage2),0)) %>% 
  dplyr::mutate(ldlc_cat=replace(ldlc_cat,stage2==0,NA)) %>% 
  dplyr::mutate(hdlc_cat2=replace(hdlc_cat2,stage2==0,NA)) 

#7- specify study design as required by "survey" library
dat.2s.a.full.design <- survey::twophase(id = list(~randid, ~randid), 
                                         subset = ~stage2, 
                                         strata = list(NULL, ~strata_cat), 
                                         data = dat.2s.a.full)

#8- run model
dat.2s.a.full.svy <- survey::svyglm(anychd ~ ldlc_cat + hdlc_cat2 + sex + prevhyp, 
                                      family = binomial("logit"), 
                                      design = dat.2s.a.full.design)

tmp1 <- data.frame(summary(dat.2s.a.full.svy)$coef)
tmp1$OR <- round(exp(tmp1[,1]),3)
tmp1$SE.log.OR <- round(tmp1[,2],3)
tmp1$lower.CI <- round(exp(tmp1[,1] - 1.96* tmp1[,2]),3)
tmp1$upper.CI <- round(exp(tmp1[,1] + 1.96* tmp1[,2]),3)
names(tmp1)[4] <- 'p-value'
knitr::kable(tmp1[,c('OR','SE.log.OR', 'lower.CI','upper.CI','p-value')])  

########--------Strategy b(for approximately 1554 sampled data)-------##########
#1- There is 183 surplus suggesting 183/6=30.5 approximately 31 extra observation
##  to sample, i.e., 195+31=226 observations to sample for 6 strata
set.seed(2307)
dat.2s.b <- dat.full_hdl %>%
  dplyr::group_by(strata_cat) %>% 
  dplyr::slice_sample(n=226, replace = FALSE)
any(duplicated(dat.2s.b$randid))

#2- assign weights for analysis not using survey library
#2a- generate the aggregated data for each stratum from full and sampled data
cbind("full"=table(dat.full_hdl$strata_cat),
      "2stage"=table(dat.2s.b$strata_cat))

dat.2s.b <- dat.2s.b %>%
  dplyr::mutate(weights=390/226) %>%
  dplyr::mutate(weights=replace(weights,strata_cat=="1.Male.0",120/120)) %>%
  dplyr::mutate(weights=replace(weights,strata_cat=="0.Female.0",631/226)) %>%
  dplyr::mutate(weights=replace(weights,strata_cat=="1.Female.0",81/81)) %>%
  dplyr::mutate(weights=replace(weights,strata_cat=="0.Male.1",479/226)) %>%
  dplyr::mutate(weights=replace(weights,strata_cat=="1.Male.1",315/226)) %>%
  dplyr::mutate(weights=replace(weights,strata_cat=="0.Female.1",749/226)) %>%
  dplyr::mutate(weights=replace(weights,strata_cat=="1.Female.1",261/226))


#3- performed weighted logistic regression not using survey library 
dat.2s.b.wt <- glm(anychd ~ ldlc_cat + hdlc_cat2 + sex + prevhyp, 
                   family = binomial("logit"), 
                   data = dat.2s.b, 
                   weights = weights)

cov.m1 <- vcovHC(dat.2s.b.wt, type = "HC0")
std.err <- sqrt(diag(cov.m1))
q.val <- qnorm(0.975)
r.est <- cbind(
  OR = round(exp(coef(dat.2s.b.wt)),3)
  , "Robust SE(log.OR)" = round(std.err,3)
  , "Lower 95% CI" = round(exp(coef(dat.2s.b.wt) - q.val  * std.err),3)
  , "Upper 95% CI" = round(exp(coef(dat.2s.b.wt) + q.val  * std.err),3)
  , "Pr(>|z|) "= round(2 * pnorm(abs(coef(dat.2s.b.wt)/std.err), lower.tail = FALSE),3))
knitr::kable(r.est, format = "pandoc",
             col.names = c("OR", "Robust SE(log.OR)", "Lower 95% CI", "Upper 95% CI", "p-value"))

#4- generate data with stage2 variable to indicate selection into the 2-stage
#   with randid variable only
dat.2s.b.select <- dat.2s.b %>%
  dplyr::mutate(stage2=1) %>%
  dplyr::select(randid,stage2,strata_cat) 

#5- merge the previous data with the full data
dat.2s.b.full <- merge(x = dat.full_hdl, y = dat.2s.b.select[,-3], 
                       by = "randid", all.x = TRUE)

#6- replace NA entries of stage2 variable as 0 and their corresponding HDL and 
#   LDL entries as missing
dat.2s.b.full <- dat.2s.b.full %>%
  dplyr::mutate(stage2=replace(stage2,is.na(stage2),0)) %>% 
  dplyr::mutate(ldlc_cat=replace(ldlc_cat,stage2==0,NA)) %>% 
  dplyr::mutate(hdlc_cat2=replace(hdlc_cat2,stage2==0,NA)) 

#7- specify study design as required by "survey" library
dat.2s.b.full.design <- survey::twophase(id = list(~randid, ~randid), 
                                         subset = ~stage2, 
                                         strata = list(NULL, ~strata_cat), 
                                         data = dat.2s.b.full)

#8- run model
dat.2s.b.full.svy <- survey::svyglm(anychd ~ ldlc_cat + hdlc_cat2 + sex + prevhyp, 
                                    family = binomial("logit"), 
                                    design = dat.2s.b.full.design)

tmp1 <- data.frame(summary(dat.2s.b.full.svy)$coef)
tmp1$OR <- round(exp(tmp1[,1]),3)
tmp1$SE.log.OR <- round(tmp1[,2],3)
tmp1$lower.CI <- round(exp(tmp1[,1] - 1.96* tmp1[,2]),3)
tmp1$upper.CI <- round(exp(tmp1[,1] + 1.96* tmp1[,2]),3)
names(tmp1)[4] <- 'p-value'
knitr::kable(tmp1[,c('OR','SE.log.OR', 'lower.CI','upper.CI','p-value')])  

#Compared to Question 1, the results are generally comparable. 

#Approach a. and b. will use different weights. An investigator might wish to take an "unbalanced" sample because investigator may be interested in certain group of people (specific strata). For example, man with history of heart disease.


#######-----Strategy b(alternative when exactly 1554 sampled data)-----#########
#Given that there are 183 surplus and 183/6=30.5 per stratum among the 6 strata
#with >195 observations. Since the smallest among the 6 can sample 31 observations,
#we let the larger 5 stratum to sample 195+31=226 observations and the smallest
#to sample 183-31*5=28 observations (i.e., 195+28=223 observations).

#1- sample 226 for the larger 5 strata. Among the 3 smaller strata they will 
#    sample 81, 120 and 223 observations
set.seed(2307)
dat.2s.b1 <- dat.full_hdl %>%
  dplyr::filter(strata_cat!="1.Female.1") %>%
  dplyr::group_by(strata_cat) %>% 
  dplyr::slice_sample(n=226, replace = FALSE)

dat.2s.b2 <- dat.full_hdl %>%
  dplyr::filter(strata_cat=="1.Female.1") %>%
  dplyr::group_by(strata_cat) %>% 
  dplyr::slice_sample(n=223, replace = FALSE)

dat.2s.b<-rbind(dat.2s.b1, dat.2s.b2)

#2- assign weights for analysis not using survey library
#2a- generate the aggregated data for each stratum from full and sampled data
cbind("full"=table(dat.full_hdl$strata_cat),
      "2stage"=table(dat.2s.b$strata_cat))

dat.2s.b <- dat.2s.b %>%
  dplyr::mutate(weights=390/226) %>%
  dplyr::mutate(weights=replace(weights,strata_cat=="1.Male.0",120/120)) %>%
  dplyr::mutate(weights=replace(weights,strata_cat=="0.Female.0",631/226)) %>%
  dplyr::mutate(weights=replace(weights,strata_cat=="1.Female.0",81/81)) %>%
  dplyr::mutate(weights=replace(weights,strata_cat=="0.Male.1",479/226)) %>%
  dplyr::mutate(weights=replace(weights,strata_cat=="1.Male.1",315/226)) %>%
  dplyr::mutate(weights=replace(weights,strata_cat=="0.Female.1",749/226)) %>%
  dplyr::mutate(weights=replace(weights,strata_cat=="1.Female.1",261/223))


#3- performed weighted logistic regression not using survey library 
dat.2s.b.wt <- glm(anychd ~ ldlc_cat + hdlc_cat2 + sex + prevhyp, 
                   family = binomial("logit"), 
                   data = dat.2s.b, 
                   weights = weights)

cov.m1 <- vcovHC(dat.2s.b.wt, type = "HC0")
std.err <- sqrt(diag(cov.m1))
q.val <- qnorm(0.975)
r.est <- cbind(
  OR = round(exp(coef(dat.2s.b.wt)),3)
  , "Robust SE(log.OR)" = round(std.err,3)
  , "Lower 95% CI" = round(exp(coef(dat.2s.b.wt) - q.val  * std.err),3)
  , "Upper 95% CI" = round(exp(coef(dat.2s.b.wt) + q.val  * std.err),3)
  , "Pr(>|z|) "= round(2 * pnorm(abs(coef(dat.2s.b.wt)/std.err), lower.tail = FALSE),3))
knitr::kable(r.est, format = "pandoc",
             col.names = c("OR", "Robust SE(log.OR)", "Lower 95% CI", "Upper 95% CI", "p-value"))

#4- generate data with stage2 variable to indicate selection into the 2-stage
#   with randid variable only
dat.2s.b.select <- dat.2s.b %>%
  dplyr::mutate(stage2=1) %>%
  dplyr::select(randid,stage2,strata_cat) 

#5- merge the previous data with the full data
dat.2s.b.full <- merge(x = dat.full_hdl, y = dat.2s.b.select[,-3], 
                       by = "randid", all.x = TRUE)

#6- replace NA entries of stage2 variable as 0 and their corresponding HDL and 
#   LDL entries as missing
dat.2s.b.full <- dat.2s.b.full %>%
  dplyr::mutate(stage2=replace(stage2,is.na(stage2),0)) %>% 
  dplyr::mutate(ldlc_cat=replace(ldlc_cat,stage2==0,NA)) %>% 
  dplyr::mutate(hdlc_cat2=replace(hdlc_cat2,stage2==0,NA)) 

#7- specify study design as required by "survey" library
dat.2s.b.full.design <- survey::twophase(id = list(~randid, ~randid), 
                                         subset = ~stage2, 
                                         strata = list(NULL, ~strata_cat), 
                                         data = dat.2s.b.full)

#8- run model
dat.2s.b.full.svy <- survey::svyglm(anychd ~ ldlc_cat + hdlc_cat2 + sex + prevhyp, 
                                    family = binomial("logit"), 
                                    design = dat.2s.b.full.design)

tmp1 <- data.frame(summary(dat.2s.b.full.svy)$coef)
tmp1$OR <- round(exp(tmp1[,1]),3)
tmp1$SE.log.OR <- round(tmp1[,2],3)
tmp1$lower.CI <- round(exp(tmp1[,1] - 1.96* tmp1[,2]),3)
tmp1$upper.CI <- round(exp(tmp1[,1] + 1.96* tmp1[,2]),3)
names(tmp1)[4] <- 'p-value'
knitr::kable(tmp1[,c('OR','SE.log.OR', 'lower.CI','upper.CI','p-value')])  
